@@ -1,8 +1,9 @@
+# rubocop:disable Naming/FileName
 # frozen_string_literal: true
 
 def require_all(path)
   dir = File.join(File.dirname(__FILE__), path)
-  Dir[File.join(dir, '*.rb')].each do |f|
+  Dir[File.join(dir, '*.rb')].sort.each do |f|
     require f
   end
 end
@@ -17,7 +18,7 @@ require 'fileutils'
 begin
   require 'awesome_print'
   require 'pry-byebug'
-rescue LoadError; end # rubocop:disable Lint/HandleExceptions
+rescue LoadError; end # rubocop:disable Lint/SuppressedException
 module HTMLProofer
   def self.check_file(file, options = {})
     raise ArgumentError unless file.is_a?(String)
@@ -52,3 +53,4 @@ module HTMLProofer
     HTMLProofer::Runner.new(links, options)
   end
 end
+# rubocop:enable Naming/FileName
