@@ -3,6 +3,7 @@
 require_relative 'utils'
 require 'date'
 require 'json'
+require 'uri'
 
 module HTMLProofer
   class Cache
@@ -11,6 +12,8 @@ module HTMLProofer
     DEFAULT_STORAGE_DIR = File.join('tmp', '.htmlproofer')
     DEFAULT_CACHE_FILE_NAME = 'cache.log'
 
+    URI_REGEXP = URI::DEFAULT_PARSER.make_regexp
+
     attr_reader :exists, :cache_log, :storage_dir, :cache_file
 
     def initialize(logger, options)
@@ -30,6 +33,8 @@ module HTMLProofer
     end
 
     def within_timeframe?(time)
+      return false if time.nil?
+
       (@parsed_timeframe..@cache_time).cover?(Time.parse(time))
     end
 
@@ -59,6 +64,8 @@ module HTMLProofer
     end
 
     def add(url, filenames, status, msg = '')
+      return unless use_cache?
+
       data = {
         time: @cache_time,
         filenames: filenames,
@@ -69,10 +76,14 @@ module HTMLProofer
       @cache_log[clean_url(url)] = data
     end
 
-    def detect_url_changes(found)
-      existing_urls = @cache_log.keys.map { |url| clean_url(url) }
+    def detect_url_changes(found, type)
       found_urls = found.keys.map { |url| clean_url(url) }
 
+      # if there were no urls, bail
+      return {} if found_urls.empty?
+
+      existing_urls = @cache_log.keys.map { |url| clean_url(url) }
+
       # prepare to add new URLs detected
       additions = found.reject do |url, _|
         url = clean_url(url)
@@ -89,24 +100,28 @@ module HTMLProofer
       @logger.log :info, "Adding #{new_link_text} to the cache..."
 
       # remove from cache URLs that no longer exist
-      del = 0
+      deletions = 0
       @cache_log.delete_if do |url, _|
         url = clean_url(url)
-        if !found_urls.include?(url)
-          @logger.log :debug, "Removing #{url} from cache check"
-          del += 1
-          true
-        else
+
+        if found_urls.include?(url)
           false
+        elsif url_matches_type?(url, type)
+          @logger.log :debug, "Removing #{url} from cache check"
+          deletions += 1
+          true
         end
       end
 
-      del_link_text = pluralize(del, 'link', 'links')
+      del_link_text = pluralize(deletions, 'link', 'links')
       @logger.log :info, "Removing #{del_link_text} from the cache..."
 
       additions
     end
 
+    # TODO: Garbage performance--both the external and internal
+    # caches need access to this file. Write a proper versioned
+    # schema in the future
     def write
       File.write(cache_file, @cache_log.to_json)
     end
@@ -115,12 +130,13 @@ module HTMLProofer
       @load.nil?
     end
 
-    def retrieve_urls(external_urls)
-      urls_to_check = detect_url_changes(external_urls)
+    def retrieve_urls(urls, type)
+      urls_to_check = detect_url_changes(urls, type)
+
       @cache_log.each_pair do |url, cache|
-        if within_timeframe?(cache['time'])
-          next if cache['message'].empty? # these were successes to skip
-        else
+        next if within_timeframe?(cache['time']) && cache['message'].empty? # these were successes to skip
+
+        if url_matches_type?(url, type)
           urls_to_check[url] = cache['filenames'] # recheck expired links
         end
       end
@@ -149,9 +165,9 @@ module HTMLProofer
 
       @cache_file = File.join(storage_dir, cache_file_name)
 
-      return unless File.exist?(cache_file)
+      return unless File.exist?(@cache_file)
 
-      contents = File.read(cache_file)
+      contents = File.read(@cache_file)
       @cache_log = contents.empty? ? {} : JSON.parse(contents)
     end
 
@@ -169,5 +185,10 @@ module HTMLProofer
         @cache_datetime - Rational(measurement / 24.0)
       end.to_time
     end
+
+    def url_matches_type?(url, type)
+      return true if type == :internal && url !~ URI_REGEXP
+      return true if type == :external && url =~ URI_REGEXP
+    end
   end
 end
