@@ -4,6 +4,8 @@ class LinkCheck < ::HTMLProofer::Check
   include HTMLProofer::Utils
 
   def missing_href?
+    return blank?(@link.src) if @node.name == 'source'
+
     blank?(@link.href) && blank?(@link.name) && blank?(@link.id)
   end
 
@@ -12,7 +14,7 @@ class LinkCheck < ::HTMLProofer::Check
   end
 
   def run
-    @html.css('a, link').each do |node|
+    @html.css('a, link, source').each do |node|
       @link = create_element(node)
       line = node.line
       content = node.to_s
@@ -34,7 +36,7 @@ class LinkCheck < ::HTMLProofer::Check
       if missing_href?
         next if @link.allow_missing_href?
         # HTML5 allows dropping the href: http://git.io/vBX0z
-        next if @html.internal_subset.name == 'html' && @html.internal_subset.external_id.nil?
+        next if @html.internal_subset.nil? || (@html.internal_subset.name == 'html' && @html.internal_subset.external_id.nil?)
 
         add_issue('anchor has no href attribute', line: line, content: content)
         next
@@ -43,31 +45,41 @@ class LinkCheck < ::HTMLProofer::Check
       # intentionally here because we still want valid? & missing_href? to execute
       next if @link.non_http_remote?
 
-      if !@link.internal? && @link.remote?
+      if !@link.href&.start_with?('#') && !@link.internal? && @link.remote?
         check_sri(line, content) if @link.check_sri? && node.name == 'link'
         # we need to skip these for now; although the domain main be valid,
         # curl/Typheous inaccurately return 404s for some links. cc https://git.io/vyCFx
         next if @link.respond_to?(:rel) && @link.rel == 'dns-prefetch'
 
-        add_to_external_urls(@link.href)
-        next
-      elsif @link.internal? && !@link.exists?
-        add_issue("internally linking to #{@link.href}, which does not exist", line: line, content: content)
-      end
+        unless @link.path?
+          add_issue("#{@link.href} is an invalid URL", line: line, content: content)
+          next
+        end
 
-      # does the local directory have a trailing slash?
-      if @link.unslashed_directory? @link.absolute_path
-        add_issue("internally linking to a directory #{@link.absolute_path} without trailing slash", line: line, content: content)
+        add_to_external_urls(@link.href || @link.src)
         next
+      elsif @link.internal?
+        add_to_internal_urls(@link.href, InternalLink.new(@link, @path, line, content))
+        add_issue("internally linking to #{@link.href}, which does not exist", line: line, content: content) if !@link.exists? && !@link.hash
       end
-
-      # verify the target hash
-      handle_hash(@link, line, content) if @link.hash
     end
 
     external_urls
   end
 
+  def check_internal_link(link, path, line, content)
+    # does the local directory have a trailing slash?
+    if link.unslashed_directory?(link.absolute_path)
+      add_issue("internally linking to a directory #{link.absolute_path} without trailing slash", path: path, line: line, content: content)
+      return false
+    end
+
+    return true unless link.hash
+
+    # verify the target hash
+    handle_hash(link, path, line, content)
+  end
+
   def check_schemes(link, line, content)
     case link.scheme
     when 'mailto'
@@ -93,24 +105,28 @@ class LinkCheck < ::HTMLProofer::Check
     add_issue("#{link.href} contains no phone number", line: line, content: content) if link.path.empty?
   end
 
-  def handle_hash(link, line, content)
-    if link.internal? && !hash_check(link.html, link.hash)
-      add_issue("linking to internal hash ##{link.hash} that does not exist", line: line, content: content)
+  def handle_hash(link, path, line, content)
+    if link.internal? && !hash_exists?(link.html, link.hash) # rubocop:disable Style/GuardClause
+      return add_issue("linking to internal hash ##{link.hash} that does not exist", path: path, line: line, content: content)
     elsif link.external?
-      external_link_check(link, line, content)
+      return external_link_check(link, line, content)
     end
+
+    true
   end
 
   def external_link_check(link, line, content)
-    if !link.exists?
-      add_issue("trying to find hash of #{link.href}, but #{link.absolute_path} does not exist", line: line, content: content)
+    if link.exists? # rubocop:disable Style/GuardClause
+      target_html = create_nokogiri(link.absolute_path)
+      return add_issue("linking to #{link.href}, but #{link.hash} does not exist", line: line, content: content) unless hash_exists?(target_html, link.hash)
     else
-      target_html = create_nokogiri link.absolute_path
-      add_issue("linking to #{link.href}, but #{link.hash} does not exist", line: line, content: content) unless hash_check target_html, link.hash
+      return add_issue("trying to find hash of #{link.href}, but #{link.absolute_path} does not exist", line: line, content: content)
     end
+
+    true
   end
 
-  def hash_check(html, href_hash)
+  def hash_exists?(html, href_hash)
     decoded_href_hash = Addressable::URI.unescape(href_hash)
     fragment_ids = [href_hash, decoded_href_hash]
     # https://www.w3.org/TR/html5/single-page.html#scroll-to-fragid
@@ -148,7 +164,19 @@ class LinkCheck < ::HTMLProofer::Check
 
   class XpathFunctions
     def case_sensitive_equals(node_set, str_to_match)
-      node_set.find_all { |node| node.to_s. == str_to_match.to_s }
+      node_set.find_all { |node| node.to_s.== str_to_match.to_s }
+    end
+  end
+
+  class InternalLink
+    attr_reader :link, :href, :path, :line, :content
+
+    def initialize(link, path, line, content)
+      @link = link
+      @href = @link.href
+      @path = path
+      @line = line
+      @content = content
     end
   end
 end
