@@ -51,7 +51,10 @@ module HTMLProofer
     VALIDATION_DEFAULTS = {
       report_script_embeds: false,
       report_missing_names: false,
-      report_invalid_tags: false
+      report_invalid_tags: false,
+      report_missing_doctype: false,
+      report_eof_tags: false,
+      report_mismatched_tags: false
     }.freeze
 
     CACHE_DEFAULTS = {}.freeze
@@ -64,7 +67,7 @@ module HTMLProofer
       end
     end
 
-    def self.parse_json_option(option_name, config)
+    def self.parse_json_option(option_name, config, symbolize_names: true)
       raise ArgumentError, 'Must provide an option name in string format.' unless option_name.is_a?(String)
       raise ArgumentError, 'Must provide an option name in string format.' if option_name.strip.empty?
 
@@ -75,9 +78,9 @@ module HTMLProofer
       return {} if config.strip.empty?
 
       begin
-        JSON.parse(config)
+        JSON.parse(config, { symbolize_names: symbolize_names })
       rescue StandardError
-        raise ArgumentError, "Option '" + option_name + "' did not contain valid JSON."
+        raise ArgumentError, "Option '#{option_name} did not contain valid JSON."
       end
     end
   end
