@@ -10,15 +10,21 @@ module HTMLProofer
 
     attr_reader :id, :name, :alt, :href, :link, :src, :line, :data_proofer_ignore
 
-    def initialize(obj, check)
+    def initialize(obj, check, logger)
+      @logger = logger
       # Construct readable ivars for every element
-      obj.attributes.each_pair do |attribute, value|
-        name = attribute.tr('-:.', '_').to_s.to_sym
-        (class << self; self; end).send(:attr_reader, name)
-        instance_variable_set("@#{name}", value.value)
+      begin
+        obj.attributes.each_pair do |attribute, value|
+          name = attribute.tr('-:.;@', '_').to_s.to_sym
+          (class << self; self; end).send(:attr_reader, name)
+          instance_variable_set("@#{name}", value.value)
+        end
+      rescue NameError => e
+        @logger.log :error, "Attribute set `#{obj}` contains an error!"
+        raise e
       end
 
-      @aria_hidden = defined?(@aria_hidden) && @aria_hidden == 'true' ? true : false
+      @aria_hidden = defined?(@aria_hidden) && @aria_hidden == 'true'
 
       @data_proofer_ignore = defined?(@data_proofer_ignore)
 
@@ -36,19 +42,19 @@ module HTMLProofer
 
       # fix up missing protocols
       if defined?(@href)
-        @href.insert(0, 'http:') if @href =~ %r{^//}
+        @href.insert(0, 'http:') if %r{^//}.match?(@href)
       else
         @href = nil
       end
 
       if defined?(@src)
-        @src.insert(0, 'http:') if @src =~ %r{^//}
+        @src.insert(0, 'http:') if %r{^//}.match?(@src)
       else
         @src = nil
       end
 
       if defined?(@srcset)
-        @srcset.insert(0, 'http:') if @srcset =~ %r{^//}
+        @srcset.insert(0, 'http:') if %r{^//}.match?(@srcset)
       else
         @srcset = nil
       end
@@ -68,6 +74,10 @@ module HTMLProofer
       !parts.nil?
     end
 
+    def path?
+      !parts.host.nil? && !parts.path.nil?
+    end
+
     def parts
       @parts ||= Addressable::URI.parse url
     rescue URI::Error, Addressable::URI::InvalidURIError
@@ -99,12 +109,10 @@ module HTMLProofer
       return true if @data_proofer_ignore
       return true if @parent_ignorable
 
-      return true if url =~ /^javascript:/
+      return true if /^javascript:/.match?(url)
 
       # ignore base64 encoded images
-      if %w[ImageCheck FaviconCheck].include? @type
-        return true if url =~ /^data:image/
-      end
+      return true if %w[ImageCheck FaviconCheck].include?(@type) && /^data:image/.match?(url)
 
       # ignore user defined URLs
       return true if ignores_pattern_check(@check.options[:url_ignore])
@@ -165,6 +173,10 @@ module HTMLProofer
       url.start_with?('?')
     end
 
+    def absolute_path?(path)
+      path.start_with?('/')
+    end
+
     def file_path
       return if path.nil? || path.empty?
 
@@ -172,22 +184,19 @@ module HTMLProofer
 
       path_dot_ext = path + @check.options[:extension] if @check.options[:assume_extension]
 
-      if path =~ %r{^/} # path relative to root
-        if File.directory?(@check.src)
-          base = @check.src
-        else
-          root_dir = @check.options[:root_dir]
-          base = root_dir || File.dirname(@check.src)
-        end
-      elsif File.exist?(File.expand_path(path, @check.src)) || File.exist?(File.expand_path(path_dot_ext, @check.src)) # relative links, path is a file
-        base = File.dirname @check.path
-      elsif File.exist?(File.join(File.dirname(@check.path), path)) || File.exist?(File.join(File.dirname(@check.path), path_dot_ext)) # relative links in nested dir, path is a file
-        base = File.dirname @check.path
-      else # relative link, path is a directory
-        base = @check.path
-      end
+      base = if absolute_path?(path) # path relative to root
+               # either overwrite with root_dir; or, if source is directory, use that; or, just get the current file's dirname
+               @check.options[:root_dir] || (File.directory?(@check.src) ? @check.src : File.dirname(@check.src))
+             elsif File.exist?(File.expand_path(path, @check.src)) || File.exist?(File.expand_path(path_dot_ext, @check.src)) # relative links, path is a file
+               File.dirname(@check.path)
+             elsif File.exist?(File.join(File.dirname(@check.path), path)) || File.exist?(File.join(File.dirname(@check.path), path_dot_ext)) # rubocop:disable Lint/DuplicateBranch; relative links in nested dir, path is a file
+               File.dirname(@check.path)
+             else # relative link, path is a directory
+               @check.path
+             end
+
+      file = File.join(base, path)
 
-      file = File.join base, path
       if @check.options[:assume_extension] && File.file?("#{file}#{@check.options[:extension]}")
         file = "#{file}#{@check.options[:extension]}"
       elsif File.directory?(file) && !unslashed_directory?(file) # implicit index support
@@ -199,22 +208,26 @@ module HTMLProofer
 
     # checks if a file exists relative to the current pwd
     def exists?
-      return @checked_paths[absolute_path] if @checked_paths.key? absolute_path
+      return @checked_paths[absolute_path] if @checked_paths.key?(absolute_path)
 
-      @checked_paths[absolute_path] = File.exist? absolute_path
+      @checked_paths[absolute_path] = File.exist?(absolute_path)
     end
 
     def absolute_path
       path = file_path || @check.path
-      File.expand_path path, Dir.pwd
+
+      File.expand_path(path, Dir.pwd)
     end
 
     def ignores_pattern_check(links)
+      return false unless links.is_a?(Array)
+
       links.each do |ignore|
-        if ignore.is_a? String
+        case ignore
+        when String
           return true if ignore == url
-        elsif ignore.is_a? Regexp
-          return true if ignore =~ url
+        when Regexp
+          return true if ignore&.match?(url)
         end
       end
 
