@@ -10,21 +10,23 @@ module HTMLProofer
     include HTMLProofer::Utils
 
     attr_reader :external_urls
+    attr_writer :before_request
 
-    def initialize(logger, external_urls, options)
+    def initialize(logger, cache, external_urls, options)
       @logger = logger
       @external_urls = external_urls
       @failed_tests = []
       @options = options
       @hydra = Typhoeus::Hydra.new(@options[:hydra])
-      @cache = Cache.new(@logger, @options[:cache])
+      @cache = cache
+      @before_request = []
     end
 
     def run
       @external_urls = remove_query_values
 
       if @cache.use_cache?
-        urls_to_check = load_cache
+        urls_to_check = @cache.retrieve_urls(@external_urls, :external)
         external_link_checker(urls_to_check)
         @cache.write
       else
@@ -41,11 +43,11 @@ module HTMLProofer
       iterable_external_urls = @external_urls.dup
       @external_urls.each_key do |url|
         uri = begin
-                Addressable::URI.parse(url)
-              rescue URI::Error, Addressable::URI::InvalidURIError
-                @logger.log :error, "#{url} is an invalid URL"
-                nil
-              end
+          Addressable::URI.parse(url)
+        rescue URI::Error, Addressable::URI::InvalidURIError
+          @logger.log :error, "#{url} is an invalid URL"
+          nil
+        end
         next if uri.nil? || uri.query.nil?
 
         iterable_external_urls.delete(url) unless new_url_query_values?(uri, paths_with_queries)
@@ -72,15 +74,6 @@ module HTMLProofer
       uri.host + uri.path
     end
 
-    def load_cache
-      cache_count = @cache.size
-      cache_text = pluralize(cache_count, 'link', 'links')
-
-      @logger.log :info, "Found #{cache_text} in the cache..."
-
-      @cache.retrieve_urls(@external_urls)
-    end
-
     # Proofer runs faster if we pull out all the external URLs and run the checks
     # at the end. Otherwise, we're halting the consuming process for every file during
     # `process_files`.
@@ -92,7 +85,7 @@ module HTMLProofer
     # for HEAD. If we've decided to check for hashes, we must do a GET--HEAD is
     # not available as an option.
     def external_link_checker(external_urls)
-      external_urls = Hash[external_urls.sort]
+      external_urls = external_urls.sort.to_h
 
       count = external_urls.length
       check_text = pluralize(count, 'external link', 'external links')
@@ -109,11 +102,11 @@ module HTMLProofer
     def establish_queue(external_urls)
       external_urls.each_pair do |url, filenames|
         url = begin
-                 clean_url(url)
-              rescue URI::Error, Addressable::URI::InvalidURIError
-                add_external_issue(filenames, "#{url} is an invalid URL")
-                next
-               end
+          clean_url(url)
+        rescue URI::Error, Addressable::URI::InvalidURIError
+          add_external_issue(filenames, "#{url} is an invalid URL")
+          next
+        end
 
         method = if hash?(url) && @options[:check_external_hash]
                    :get
@@ -127,16 +120,19 @@ module HTMLProofer
     def clean_url(href)
       # catch any obvious issues, like strings in port numbers
       parsed = Addressable::URI.parse(href)
-      if href !~ /^([!#{$&}-;=?-\[\]_a-z~]|%[0-9a-fA-F]{2})+$/
-        parsed.normalize
-      else
+      if href =~ /^([!#{Regexp.last_match(0)}-;=?-\[\]_a-z~]|%[0-9a-fA-F]{2})+$/
         href
+      else
+        parsed.normalize
       end
     end
 
     def queue_request(method, href, filenames)
       opts = @options[:typhoeus].merge(method: method)
       request = Typhoeus::Request.new(href, opts)
+      @before_request.each do |callback|
+        callback.call(request)
+      end
       request.on_complete { |response| response_handler(response, filenames) }
       @hydra.queue request
     end
@@ -146,7 +142,7 @@ module HTMLProofer
       href = response.request.base_url.to_s
       method = response.request.options[:method]
       response_code = response.code
-      response.body.gsub!("\x00", '')
+      response.body.delete!("\x00")
 
       debug_msg = if filenames.nil?
                     "Received a #{response_code} for #{href}"
