@@ -1,11 +1,11 @@
 # frozen_string_literal: true
 
-require 'nokogiri'
+require 'nokogumbo'
 
 module HTMLProofer
   module Utils
     def pluralize(count, single, plural)
-      "#{count} #{(count == 1 ? single : plural)}"
+      "#{count} #{count == 1 ? single : plural}"
     end
 
     def create_nokogiri(path)
@@ -15,7 +15,7 @@ module HTMLProofer
                   path
                 end
 
-      Nokogiri::HTML(clean_content(content))
+      Nokogiri::HTML5(content, max_errors: -1)
     end
 
     def swap(href, replacement)
@@ -24,14 +24,5 @@ module HTMLProofer
       end
       href
     end
-
-    # address a problem with Nokogiri's parsing URL entities
-    # problem from http://git.io/vBYU1
-    # solution from http://git.io/vBYUi
-    def clean_content(string)
-      string.gsub(%r{(?:https?:)?//([^>]+)}i) do |url|
-        url.gsub(/&(?!amp;)/, '&amp;')
-      end
-    end
   end
 end
